@@ -1,15 +1,14 @@
 #pragma once
 #include "Response.h"
 #include <panda/memory.h>
-#include <panda/uri/URI.h>
+#include <panda/string_map.h>
 
 namespace panda { namespace protocol { namespace http {
 
-using panda::uri::URI;
-using panda::uri::URISP;
 
 struct Request : Message, AllocatedObject<Request> {
-    enum class Method {OPTIONS, GET, HEAD, POST, PUT, DELETE, TRACE, CONNECT};
+    enum class Method {unspecified, OPTIONS, GET, HEAD, POST, PUT, DELETE, TRACE, CONNECT};
+    enum class EncType {MULTIPART, URLENCODED, disabled};
 
     static inline string method_str(Request::Method rm) noexcept {
         using Method = Request::Method;
@@ -29,9 +28,31 @@ struct Request : Message, AllocatedObject<Request> {
     struct Builder; template <class, class> struct BuilderImpl;
     using Cookies = Fields<string, true, 3>;
 
-    Method  method = Method::GET;
+    struct Form: string_multimap<string, string> {
+
+        Form(EncType enc_type = EncType::disabled) noexcept :_enc_type(enc_type){}
+
+        void enc_type (const EncType value) noexcept { _enc_type = value; }
+        EncType enc_type () const noexcept { return _enc_type; }
+
+        operator bool () const noexcept { return _enc_type != EncType::disabled; }
+
+        void add(const string& key, const string& value) {
+            insert({key, value});
+        }
+
+    private:
+        void to_body (Body& body, URI& uri, const URISP original_uri, const string& boundary) const noexcept;
+        void to_uri  (URI& uri, const URISP original_uri) const noexcept;
+        EncType _enc_type = EncType::MULTIPART;
+        friend struct Request;
+    };
+
+
+    Method  method = Method::unspecified;
     URISP   uri;
     Cookies cookies;
+    Form    form;
 
     compression::storage_t compression_prefs = Compression::IDENTITY;
 
@@ -44,8 +65,8 @@ struct Request : Message, AllocatedObject<Request> {
 
     bool expects_continue () const;
 
-    std::vector<string> to_vector ();
-    string              to_string () { return Message::to_string(to_vector()); }
+    std::vector<string> to_vector () const;
+    string              to_string () const { return Message::to_string(to_vector()); }
 
     virtual ResponseSP new_response () const { return make_iptr<Response>(); }
 
@@ -70,7 +91,8 @@ protected:
 private:
     friend struct RequestParser;
 
-    string http_header (Compression::Type applied_compression) const;
+    static Method deduce_method (bool has_form, EncType form_enc, Method method) noexcept;
+    string _http_header (SerializationContext &ctx) const;
 };
 using RequestSP = iptr<Request>;
 
@@ -103,6 +125,12 @@ struct Request::BuilderImpl : Message::Builder<T, R> {
         this->_message->allow_compression(prefn...);
         return this->self();
     }
+
+    template<typename Form>
+    T& form(Form&& form) {
+        this->_message->form = std::forward<Form>(form);
+        return this->self();
+    }
 };
 
 struct Request::Builder : Request::BuilderImpl<Builder, RequestSP> {
